package com.github.itzilly.sbt

import com.github.itzilly.sbt.features.ChestEsp
import com.github.itzilly.sbt.features.GemstoneFinder
import net.minecraft.command.CommandBase
import net.minecraft.command.CommandException
import net.minecraft.command.ICommandSender
import net.minecraft.util.ChatComponentText
import net.minecraftforge.common.config.Configuration.*


class SBTCommand : CommandBase() {

    override fun getCommandName(): String {
        return "sbt"
    }

    override fun getCommandUsage(sender: ICommandSender): String {
        return "/sbt <powderchests/gemstonesearchradius> [enable/disable/radius]"
    }

    override fun getRequiredPermissionLevel(): Int {
        return 0
    }

    @Throws(CommandException::class)
    override fun processCommand(sender: ICommandSender, args: Array<String>) {
        if (args.isEmpty()) {
            sender.addChatMessage(ChatComponentText("Invalid command. Usage: /sbt <powderchests/gemstonesearchradius> [enable/disable/radius]"))
            return
        }

        when (args[0].lowercase()) {
            "powderchests" -> {
                if (args.size < 2) {
                    sender.addChatMessage(ChatComponentText("Usage: /sbt powderchests <enable/disable>"))
                    return
                }

                when (args[1].lowercase()) {
                    "enable" -> {
                        ChestEsp.enabled = true
                        SkyBlockTweaks.config.getCategory(CATEGORY_GENERAL)?.get("powderchestenable")?.set(true)
                        SkyBlockTweaks.config.save()
                        sender.addChatMessage(ChatComponentText("Powder ChestESP enabled."))
                    }
                    "disable" -> {
                        ChestEsp.enabled = false
                        SkyBlockTweaks.config.getCategory(CATEGORY_GENERAL)?.get("powderchestenable")?.set(false)
                        SkyBlockTweaks.config.save()
                        sender.addChatMessage(ChatComponentText("Powder ChestESP disabled."))
                    }
                    else -> {
                        sender.addChatMessage(ChatComponentText("Invalid option. Usage: /sbt powderchests <enable/disable>"))
                    }
                }
            }
            "gemstonesearchradius" -> {
                if (args.size < 2) {
                    sender.addChatMessage(ChatComponentText("Usage: /sbt gemstonesearchradius <radius>"))
                    return
                }

                try {
                    val radius = args[1].toInt()

                    if (radius < 0) {
                        sender.addChatMessage(ChatComponentText("Radius must be a positive integer."))
                        return
                    }

                    SkyBlockTweaks.config.getCategory(CATEGORY_GENERAL)?.get("gemstonesearchradius")?.set(radius)
                    SkyBlockTweaks.config.save()
                    GemstoneFinder.radius = radius
                    sender.addChatMessage(ChatComponentText("Gemstone search radius set to $radius."))

                } catch (e: NumberFormatException) {
                    sender.addChatMessage(ChatComponentText("Invalid radius. Please provide a positive integer."))
                }
            }
            else -> {
                sender.addChatMessage(ChatComponentText("Invalid command. Usage: /sbt <powderchests/gemstonesearchradius> [enable/disable/radius]"))
            }
        }
    }
}