package com.github.itzilly.sbt

import com.github.itzilly.sbt.features.ChestEsp
import com.github.itzilly.sbt.features.FarmHelper
import com.github.itzilly.sbt.features.GemstoneFinder
import com.github.itzilly.sbt.features.GrottoFinder
import com.github.itzilly.sbt.features.router.RouteBuilder
import com.github.itzilly.sbt.util.MessageScheduler
import net.minecraftforge.client.ClientCommandHandler
import net.minecraftforge.common.MinecraftForge
import net.minecraftforge.common.config.Configuration
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.event.FMLInitializationEvent
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent
import java.io.File

@Mod(modid = "sbt", useMetadata = true, clientSideOnly = true)
class SkyBlockTweaks {
    init {
        INSTANCE = this
    }

    companion object {
        lateinit var INSTANCE: SkyBlockTweaks
        lateinit var config: Configuration
    }

    @Mod.EventHandler
    fun preInit(event: FMLPreInitializationEvent) {
        val directory = event.modConfigurationDirectory
        config = Configuration(File(directory.path, "sbt.cfg"))
        loadConfig()
    }

    @Mod.EventHandler
    fun init(event: FMLInitializationEvent) {
        STBKeyBinds.init()

        MinecraftForge.EVENT_BUS.register(RouteBuilder)
        MinecraftForge.EVENT_BUS.register(InputHandler())
        MinecraftForge.EVENT_BUS.register(MessageScheduler)
        MinecraftForge.EVENT_BUS.register(GrottoFinder)
        MinecraftForge.EVENT_BUS.register(ChestEsp)
        MinecraftForge.EVENT_BUS.register(GemstoneFinder)
        MinecraftForge.EVENT_BUS.register(FarmHelper)

        ClientCommandHandler.instance.registerCommand(SBTCommand())
    }

    private fun loadConfig() {
        config.load()

        val powderChestEnable = config.getBoolean(
            "powderchestenable",
            Configuration.CATEGORY_GENERAL,
            true,
            "Enables/Disables Powder ChestESP"
        )
        ChestEsp.enabled = powderChestEnable

        val gemstoneRadius = config.getInt(
            "gemstonesearchradius",
            Configuration.CATEGORY_GENERAL,
            16,
            1,
            256,
            "Enables/Disables Powder ChestESP"
        )
        ChestEsp.enabled = powderChestEnable
        GemstoneFinder.radius = gemstoneRadius

        if (config.hasChanged()) {
            config.save()
        }
    }

    fun syncConfig() {
        loadConfig()
        if (config.hasChanged()) {
            config.save()
        }
    }
}