package com.github.itzilly.sbt.features.gardener

import com.github.itzilly.sbt.mixin.KeyBindingAccessor
import com.github.itzilly.sbt.util.Chatterbox
import net.minecraft.client.Minecraft
import net.minecraft.client.gui.GuiChat
import net.minecraft.client.settings.KeyBinding
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.common.gameevent.TickEvent
import net.minecraftforge.fml.common.gameevent.TickEvent.ClientTickEvent


object CaneMacroer {
    var isMacroToggled: Boolean = false
    val mc = Minecraft.getMinecraft()
    val allowedScreens = listOf(GuiChat::class.java)

    @SubscribeEvent
    fun onTick(event: ClientTickEvent) {
        if (event.phase != TickEvent.Phase.END) return
        if (!isMacroToggled) return

        val currentScreen = mc.currentScreen

        if (currentScreen != null && allowedScreens.none { it.isInstance(currentScreen) }) {
            Chatterbox.say("Macro stopped: Invalid screen")
            stop()
            return
        }

        val mop = mc.objectMouseOver

        if (mop == null || mop.typeOfHit?.name != "BLOCK") return

        val blockPos = mop.blockPos
        val facing = mop.sideHit

        val player = mc.thePlayer
        val blockCenter = blockPos.add(0.5, 0.5, 0.5)
        val distanceSq = player.getDistanceSq(blockCenter.x.toDouble(), blockCenter.y.toDouble(), blockCenter.z.toDouble())

        if (distanceSq > 25) {
            Chatterbox.say("Macro stopped: Too far from block")
            stop()
            return
        }

        if (isEntityBlockingView()) {
            Chatterbox.say("Macro stopped: Entity in the way")
            stop()
            return
        }

        pressKey(mc.gameSettings.keyBindForward.keyCode)
        mc.playerController.onPlayerDamageBlock(blockPos, facing)
        player.swingItem()
    }

    fun stop() {
        KeyBinding.setKeyBindState(mc.gameSettings.keyBindForward.keyCode, false)
        KeyBinding.setKeyBindState(mc.gameSettings.keyBindAttack.keyCode, false)
        isMacroToggled = false
    }

    private fun pressKey(keyCode: Int) {
        KeyBinding.setKeyBindState(keyCode, true)
    }

    private fun isEntityBlockingView(): Boolean {
        val lookVec = mc.thePlayer.lookVec
        val eyePos = mc.thePlayer.getPositionEyes(1f)

        val entities = mc.theWorld.getEntitiesWithinAABBExcludingEntity(
            mc.thePlayer,
            mc.thePlayer.entityBoundingBox.expand(lookVec.xCoord * 5, lookVec.yCoord * 5, lookVec.zCoord * 5)
        )

        return entities.any {
            it.canBeCollidedWith() &&
                    it.entityBoundingBox.isVecInside(eyePos)
        }
    }
}