package com.github.itzilly.sbt.renderer

import com.github.itzilly.sbt.util.Vecd3
import net.minecraft.block.Block
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.GlStateManager
import net.minecraft.entity.Entity
import net.minecraft.util.AxisAlignedBB
import net.minecraft.util.BlockPos
import net.minecraft.util.EnumFacing
import org.lwjgl.opengl.GL11
import java.awt.Color

object RenderFuncs {

    fun drawBlockOutline(entity: Entity, blockPos: BlockPos, partialTicks: Float, outlineColor: Int) {
        val padding = 0.0020000000949949026

        val entityX = entity.lastTickPosX + (entity.posX - entity.lastTickPosX) * partialTicks
        val entityY = entity.lastTickPosY + (entity.posY - entity.lastTickPosY) * partialTicks
        val entityZ = entity.lastTickPosZ + (entity.posZ - entity.lastTickPosZ) * partialTicks

        val world = Minecraft.getMinecraft().theWorld
        val blockState = world.getBlockState(blockPos)
        val block = blockState.block

        val boundingBox = block.getCollisionBoundingBox(world, blockPos, blockState)
            ?: AxisAlignedBB(
                blockPos.x.toDouble(), blockPos.y.toDouble(), blockPos.z.toDouble(),
                blockPos.x + 1.0, blockPos.y + 1.0, blockPos.z + 1.0
            ).expand(padding, padding, padding)

        // Convert integer color to r, g, b, a values
        val r = (outlineColor shr 16 and 0xFF) / 255.0f
        val g = (outlineColor shr 8 and 0xFF) / 255.0f
        val b = (outlineColor and 0xFF) / 255.0f
        val a = (outlineColor shr 24 and 0xFF) / 255.0f

        GL11.glPushMatrix()
        GlStateManager.disableTexture2D()
        GlStateManager.depthMask(false)
        GlStateManager.disableDepth()
        GL11.glEnable(GL11.GL_BLEND)
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA)
        GL11.glEnable(GL11.GL_LINE_SMOOTH)
        GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_NICEST)
        GL11.glLineWidth(2.0f)

        // Set the color with alpha
        GL11.glColor4f(r, g, b, a)

        // Draw the outline using the bounding box
        RenderUtils.drawBlock(boundingBox.offset(-entityX, -entityY, -entityZ), outlineColor, outlineColor)

        GL11.glDisable(GL11.GL_LINE_SMOOTH)
        GL11.glDisable(GL11.GL_BLEND)
        GlStateManager.enableDepth()
        GlStateManager.depthMask(true)
        GlStateManager.enableAlpha()
        GlStateManager.enableTexture2D()
        GL11.glPopMatrix()
    }

    fun drawBlockOutline(entity: Entity, blockPos: BlockPos, partialTicks: Float) {
        val padding: Double = 0.0020000000949949026

        val entityX: Double = entity.lastTickPosX + (entity.posX - entity.lastTickPosX) * partialTicks
        val entityY: Double = entity.lastTickPosY + (entity.posY - entity.lastTickPosY) * partialTicks
        val entityZ: Double = entity.lastTickPosZ + (entity.posZ - entity.lastTickPosZ) * partialTicks

        val outlineStartColor = 0xFFFFFF
        val outlineEndColor = 0xFFFFFF

        val world = Minecraft.getMinecraft().theWorld
        val blockState = world.getBlockState(blockPos)
        val block: Block = blockState.block

        // Create a default bounding box for blocks that don't have one
        val boundingBox: AxisAlignedBB = block.getCollisionBoundingBox(world, blockPos, blockState)
            ?: AxisAlignedBB(
                blockPos.x.toDouble(), blockPos.y.toDouble(), blockPos.z.toDouble(),
                blockPos.x + 1.0, blockPos.y + 1.0, blockPos.z + 1.0
            ).expand(padding, padding, padding)

        GL11.glPushMatrix()
        GlStateManager.disableTexture2D()
        GlStateManager.depthMask(false)
        GlStateManager.disableDepth()
        GL11.glEnable(GL11.GL_LINE_SMOOTH)
        GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_NICEST)
        GL11.glLineWidth(2.0f)
        GL11.glShadeModel(GL11.GL_SMOOTH)

        RenderUtils.drawBlock(boundingBox.offset(-entityX, -entityY, -entityZ), outlineStartColor, outlineEndColor)

        GL11.glLineWidth(2.0f)
        GL11.glDisable(GL11.GL_LINE_SMOOTH)
        GlStateManager.enableDepth()
        GlStateManager.depthMask(true)
        GlStateManager.enableAlpha()
        GlStateManager.enableTexture2D()
        GL11.glPopMatrix()
    }

    fun drawBlockLine(pos: BlockPos, partialTicks: Float) {
        val render = Minecraft.getMinecraft().renderViewEntity
        val rm = Minecraft.getMinecraft().renderManager
        // TODO: Fix this not working with view bobbing enabled
        val pos1 = Vecd3(rm.viewerPosX, rm.viewerPosY + render.eyeHeight, rm.viewerPosZ)

        val pos2 = Vecd3(
            pos.x.toDouble() + 0.5, // Center of the block
            pos.y.toDouble() + 0.5, // Center of the block
            pos.z.toDouble() + 0.5  // Center of the block
        )

        val lineColor = Color(0xe310d5)
        RenderUtils.drawLine(pos1, pos2, lineColor, 2, false, partialTicks)
    }

    fun drawClusterOutline(entity: Entity, positions: List<BlockPos>, partialTicks: Float, outlineColor: Int) {
        val padding = 0.0020000000949949026

        // Correctly calculate the player's interpolated position
        val entityX = entity.lastTickPosX + (entity.posX - entity.lastTickPosX) * partialTicks
        val entityY = entity.lastTickPosY + (entity.posY - entity.lastTickPosY) * partialTicks
        val entityZ = entity.lastTickPosZ + (entity.posZ - entity.lastTickPosZ) * partialTicks

        val world = Minecraft.getMinecraft().theWorld

        GL11.glPushMatrix()
        GlStateManager.disableTexture2D()
        GlStateManager.depthMask(false)
        GlStateManager.disableDepth()
        GL11.glEnable(GL11.GL_BLEND)
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA)
        GL11.glEnable(GL11.GL_LINE_SMOOTH)
        GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_NICEST)
        GL11.glLineWidth(2.0f)

        // Convert integer color to r, g, b, a values
        val r = (outlineColor shr 16 and 0xFF) / 255.0f
        val g = (outlineColor shr 8 and 0xFF) / 255.0f
        val b = (outlineColor and 0xFF) / 255.0f
        val a = (outlineColor shr 24 and 0xFF) / 255.0f

        // Set the color with alpha
        GL11.glColor4f(r, g, b, a)

        // Iterate over the positions and draw the outline around each block
        for (pos in positions) {
            val blockState = world.getBlockState(pos)
            val boundingBox = blockState.block.getCollisionBoundingBox(world, pos, blockState)
                ?: AxisAlignedBB(pos.x.toDouble(), pos.y.toDouble(), pos.z.toDouble(), pos.x + 1.0, pos.y + 1.0, pos.z + 1.0).expand(padding, padding, padding)

            // Offset the bounding box by the interpolated player position
            val offsetBoundingBox = boundingBox.offset(-entityX, -entityY, -entityZ)

            // Check adjacent blocks and only draw faces that are exposed (not adjacent to another gemstone block)
            if (!positions.contains(pos.add(1, 0, 0))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.EAST, outlineColor)
            }
            if (!positions.contains(pos.add(-1, 0, 0))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.WEST, outlineColor)
            }
            if (!positions.contains(pos.add(0, 1, 0))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.UP, outlineColor)
            }
            if (!positions.contains(pos.add(0, -1, 0))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.DOWN, outlineColor)
            }
            if (!positions.contains(pos.add(0, 0, 1))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.SOUTH, outlineColor)
            }
            if (!positions.contains(pos.add(0, 0, -1))) {
                RenderUtils.drawBlockFace(offsetBoundingBox, EnumFacing.NORTH, outlineColor)
            }
        }

        GL11.glDisable(GL11.GL_LINE_SMOOTH)
        GL11.glDisable(GL11.GL_BLEND)
        GlStateManager.enableDepth()
        GlStateManager.depthMask(true)
        GlStateManager.enableAlpha()
        GlStateManager.enableTexture2D()
        GL11.glPopMatrix()
    }
}